import React, { useState } from 'react';
import { useDispatch } from 'react-redux';
import { useNavigate } from 'react-router-dom';
import { changePasswordThunk } from '../store/auth/auth.thunk';

function ChangePasswordModal({ show, onClose }) {
    const dispatch = useDispatch();
    const navigate = useNavigate();

    const [formData, setFormData] = useState({
        old_password: '',
        new_password: '',
        confirm_password: '',
    });

    const [errors, setErrors] = useState({});
    const [isSubmitting, setIsSubmitting] = useState(false);
    const [submitted, setSubmitted] = useState(false);

    const handleChange = (e) => {
        const { name, value } = e.target;
        setFormData({
            ...formData,
            [name]: value.trim(),
        });
    };

    const validateForm = () => {
        const newErrors = {};

        if (!formData.old_password) {
            newErrors.old_password = '请输入当前密码';
        }

        if (!formData.new_password) {
            newErrors.new_password = '请输入新密码';
        } else if (formData.new_password.length < 6) {
            newErrors.new_password = '新密码长度不能少于6个字符';
        }

        if (!formData.confirm_password) {
            newErrors.confirm_password = '请确认新密码';
        } else if (formData.new_password !== formData.confirm_password) {
            newErrors.confirm_password = '两次输入的密码不一致';
        }

        setErrors(newErrors);
        return Object.keys(newErrors).length === 0;
    };

    const handleSubmit = async (e) => {
        e.preventDefault();
        setSubmitted(true);

        if (validateForm()) {
            setIsSubmitting(true);
            try {
                await dispatch(
                    changePasswordThunk({
                        old_password: formData.old_password,
                        new_password: formData.new_password,
                    })
                ).unwrap();

                // 成功后会由thunk中的代码重定向到登录页
                navigate('/login');
            } catch (error) {
                // 错误处理已经在thunk中完成
                console.error('Change password failed:', error);
            } finally {
                setIsSubmitting(false);
            }
        }
    };

    if (!show) return null;

    return (
        <div className='modal show d-block' style={{ backgroundColor: 'rgba(0,0,0,0.5)' }}>
            <div className='modal-dialog modal-dialog-centered'>
                <div className='modal-content'>
                    <div className='modal-header border-0'>
                        <h5 className='modal-title'>修改密码</h5>
                        <button type='button' className='btn-close' onClick={onClose}></button>
                    </div>
                    <div className='modal-body'>
                        <form onSubmit={handleSubmit} noValidate>
                            <div className='mb-3'>
                                <label className='form-label'>当前密码</label>
                                <input
                                    value={formData.old_password}
                                    name='old_password'
                                    type='password'
                                    className={`form-control${submitted && errors.old_password ? ' is-invalid' : ''}`}
                                    required
                                    onChange={handleChange}
                                />
                                {submitted && errors.old_password && (
                                    <div className='invalid-feedback'>{errors.old_password}</div>
                                )}
                            </div>

                            <div className='mb-3'>
                                <label className='form-label'>新密码</label>
                                <input
                                    value={formData.new_password}
                                    name='new_password'
                                    type='password'
                                    className={`form-control${submitted && errors.new_password ? ' is-invalid' : ''}`}
                                    required
                                    onChange={handleChange}
                                />
                                {submitted && errors.new_password && (
                                    <div className='invalid-feedback'>{errors.new_password}</div>
                                )}
                            </div>

                            <div className='mb-3'>
                                <label className='form-label'>确认新密码</label>
                                <input
                                    value={formData.confirm_password}
                                    name='confirm_password'
                                    type='password'
                                    className={`form-control${
                                        submitted && errors.confirm_password ? ' is-invalid' : ''
                                    }`}
                                    required
                                    onChange={handleChange}
                                />
                                {submitted && errors.confirm_password && (
                                    <div className='invalid-feedback'>{errors.confirm_password}</div>
                                )}
                            </div>

                            <div className='modal-footer border-0'>
                                <button type='button' className='btn btn-outline-dark' onClick={onClose}>
                                    取消
                                </button>
                                <button type='submit' className='btn btn-dark' disabled={isSubmitting}>
                                    {isSubmitting ? '提交中...' : '修改密码'}
                                </button>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    );
}

export default ChangePasswordModal;
