import React, { useState, useEffect, useRef } from 'react';
import ReactMarkdown from 'react-markdown';
import { Prism as SyntaxHighlighter } from 'react-syntax-highlighter';
import { vscDarkPlus } from 'react-syntax-highlighter/dist/esm/styles/prism';
import remarkGfm from 'remark-gfm';
import './SafeMarkdown.scss';
import ErrorBoundary from './ErrorBoundary';
import CodeBlock from './CodeBlock';
import SvgIcon from './SvgIcon';

/**
 * SafeMarkdown component that wraps ReactMarkdown with error handling
 * Displays raw content as fallback if markdown parsing fails
 */
const SafeMarkdown = ({ content, isStreaming = false }) => {
    const [copied, setCopied] = useState(false);
    const [animatedContent, setAnimatedContent] = useState('');
    const [typingSpeed, setTypingSpeed] = useState(10); // 打字速度(毫秒)
    const prevContentRef = useRef('');
    const isStreamingRef = useRef(isStreaming);

    // 复制代码按钮处理函数
    const handleCopy = (code) => {
        navigator.clipboard.writeText(code).then(
            () => {
                setCopied(true);
                setTimeout(() => setCopied(false), 2000);
            },
            (err) => {
                console.error('无法复制内容: ', err);
            }
        );
    };

    // 获取随机打字速度 - 使打字效果看起来更自然
    const getRandomTypingSpeed = () => {
        // 基础速度 + 随机变化
        const baseSpeed = 20;
        const variance = Math.random() * 20; // 0-20ms的随机变化

        // 如果是标点符号，稍微停顿长一点
        const nextChar = content.charAt(animatedContent.length);
        const isPunctuation = ['.', ',', '!', '?', ';', ':', '\n'].includes(nextChar);

        return isPunctuation ? baseSpeed + variance + 30 : baseSpeed + variance;
    };

    // 打字机效果
    useEffect(() => {
        // 更新流式状态引用
        isStreamingRef.current = isStreaming;

        // 如果内容是空，直接清空动画内容
        if (!content) {
            setAnimatedContent('');
            prevContentRef.current = '';
            return;
        }

        // 如果没有流式传输，或者内容已经完全相同，直接显示完整内容
        if (!isStreaming || content === animatedContent) {
            setAnimatedContent(content);
            prevContentRef.current = content;
            return;
        }

        // 如果内容比当前动画内容长，逐步添加
        if (content.length > animatedContent.length) {
            // 计算打字速度 - 根据内容长度动态调整
            let speed = getRandomTypingSpeed();
            if (content.length > 500) {
                speed = speed * 0.5; // 内容很长时加速
            } else if (content.length > 200) {
                speed = speed * 0.7; // 内容较长时稍微加速
            }

            const timeout = setTimeout(() => {
                // 根据内容长度，决定一次添加多少字符，提高长文本的打字速度
                let charsToAdd = 1;
                if (content.length > 1000) {
                    charsToAdd = 3; // 对于非常长的内容，一次添加3个字符
                } else if (content.length > 500) {
                    charsToAdd = 2; // 对于较长的内容，一次添加2个字符
                }

                const endIndex = Math.min(animatedContent.length + charsToAdd, content.length);
                const nextChars = content.substring(animatedContent.length, endIndex);

                setAnimatedContent((prev) => prev + nextChars);
            }, speed);

            return () => clearTimeout(timeout);
        }

        // 保存前一次的内容引用，用于检测内容变化
        prevContentRef.current = content;
    }, [content, animatedContent, isStreaming]);

    // Fallback UI that shows raw content when ReactMarkdown fails
    const renderFallback = (error) => {
        console.error('Markdown rendering error:', error);
        return (
            <div className='markdown-fallback'>
                <div className='text-danger'>Markdown渲染错误</div>
                <pre>{content}</pre>
            </div>
        );
    };

    try {
        // 如果内容为空，直接返回空
        if (!content) return null;

        // 实际显示的内容 - 当流式传输时使用动画内容，否则使用完整内容
        const displayContent = content;

        return (
            <ErrorBoundary fallback={renderFallback}>
                <ReactMarkdown
                    remarkPlugins={[remarkGfm]}
                    components={{
                        // Apply the className to the root element
                        root: ({ node, ...props }) => <div className='markdown-content' {...props} />,
                        code({ node, inline, className, children, ...props }) {
                            const match = /language-(\w+)/.exec(className || '');
                            const language = match && match[1] ? match[1] : '';

                            return !inline && match ? (
                                <div className='code-block-container'>
                                    <div className='code-block-header'>
                                        <span className='code-language-badge'>{language}</span>
                                        {copied ? (
                                            <span className='copied-indicator'>
                                                <SvgIcon className='check' width='14' height='14' />
                                                <span>已复制</span>
                                            </span>
                                        ) : (
                                            <button
                                                className='copy-button'
                                                onClick={() => handleCopy(String(children).replace(/\n$/, ''))}
                                            >
                                                <SvgIcon className='copy' width='14' height='14' />
                                                <span>复制</span>
                                            </button>
                                        )}
                                    </div>
                                    <SyntaxHighlighter style={vscDarkPlus} language={language} PreTag='div' {...props}>
                                        {String(children).replace(/\n$/, '')}
                                    </SyntaxHighlighter>
                                </div>
                            ) : (
                                <code className={className} {...props}>
                                    {children}
                                </code>
                            );
                        },
                    }}
                >
                    {displayContent}
                </ReactMarkdown>
                {isStreaming && animatedContent !== content && <span className='typing-cursor'></span>}
            </ErrorBoundary>
        );
    } catch (error) {
        console.error('Markdown渲染错误:', error);
        // 降级处理，纯文本显示
        return (
            <div className='markdown-fallback'>
                <div className='text-danger'>Markdown渲染错误</div>
                <pre>{content}</pre>
            </div>
        );
    }
};

export default SafeMarkdown;
