import React, { useState, useEffect, useRef } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { resetSendMessageStatus, addMessage, updateMessage } from '../../store/chat/chat.slice';
import { showNotification } from '../../store/notification.slice';
import { fetchAvailableDatasets, fetchConversationDetail, getChatResources } from '../../store/chat/chat.thunks';
import SvgIcon from '../../components/SvgIcon';
import SafeMarkdown from '../../components/SafeMarkdown';
import ResourceList from '../../components/ResourceList';
import { sendChatMessageViaWebSocket } from '../../services/websocket';

export default function ChatWindow({ chatId, knowledgeBaseId }) {
    const dispatch = useDispatch();
    const [inputMessage, setInputMessage] = useState('');
    const [loading, setLoading] = useState(false);
    const messagesEndRef = useRef(null);
    const hasLoadedDetailRef = useRef({}); // 添加ref来跟踪已加载的会话

    // 从 Redux store 获取消息
    const chatList = useSelector((state) => state.chat.list.items);
    const currentChatId = useSelector((state) => state.chat.currentChat.conversationId || chatId);
    const currentChat = chatList.find((chat) => chat.conversation_id === currentChatId);
    const messages = currentChat?.messages || [];
    const messageStatus = useSelector((state) => state.chat.list.messageStatus);
    const { status: sendStatus, error: sendError } = useSelector((state) => state.chat.sendMessage);

    // 获取消息资源
    const resources = useSelector((state) => state.chat.resources);

    // 使用新的Redux状态结构
    const knowledgeBases = useSelector((state) => state.knowledgeBase.knowledgeBases || []);
    const knowledgeBase = knowledgeBases.find((kb) => kb.id === knowledgeBaseId);

    // 获取可用数据集列表
    const availableDatasets = useSelector((state) => state.chat.availableDatasets.items || []);
    const availableDatasetsLoading = useSelector((state) => state.chat.availableDatasets.status === 'loading');

    // 获取会话详情
    const conversation = useSelector((state) => state.chat.currentChat.data);
    const conversationStatus = useSelector((state) => state.chat.currentChat.status);
    const conversationError = useSelector((state) => state.chat.currentChat.error);

    // 获取会话创建状态
    const createSessionStatus = useSelector((state) => state.chat.createSession?.status);
    const createSessionId = useSelector((state) => state.chat.createSession?.sessionId);

    // 监听知识库ID变更，确保保存在组件状态中
    const [selectedKnowledgeBaseIds, setSelectedKnowledgeBaseIds] = useState([]);

    // 当chatId改变时设置当前会话ID
    useEffect(() => {
        if (chatId) {
            // 通过设置currentChat.conversationId确保消息显示在正确的会话下
            dispatch({
                type: 'chat/setCurrentChat',
                payload: { conversation_id: chatId },
            });
        }
    }, [chatId, dispatch]);

    // 当conversation或knowledgeBaseId更新时，更新selectedKnowledgeBaseIds
    useEffect(() => {
        // 优先使用conversation中的知识库列表
        if (conversation && conversation.datasets && conversation.datasets.length > 0) {
            const datasetIds = conversation.datasets.map((ds) => ds.id);
            setSelectedKnowledgeBaseIds(datasetIds);
        }
        // 其次使用URL中传入的知识库ID
        else if (knowledgeBaseId) {
            // 可能是单个ID或以逗号分隔的多个ID
            const ids = knowledgeBaseId.split(',').map((id) => id.trim());
            setSelectedKnowledgeBaseIds(ids);
        }
    }, [conversation, knowledgeBaseId]);

    // 获取聊天详情
    useEffect(() => {
        if (!chatId) return;

        // 如果已经加载过这个chatId的详情，不再重复加载
        if (hasLoadedDetailRef.current[chatId]) {
            return;
        }

        // 检查是否是新创建的会话
        const isNewlyCreatedChat = createSessionStatus === 'succeeded' && createSessionId === chatId;

        // 如果是新创建的会话且已经有会话数据，则跳过详情获取
        if (isNewlyCreatedChat && conversation && conversation.conversation_id === chatId) {
            hasLoadedDetailRef.current[chatId] = true;
            return;
        }

        setLoading(true);

        dispatch(fetchConversationDetail(chatId))
            .unwrap()
            .then((response) => {
                // 标记为已加载
                hasLoadedDetailRef.current[chatId] = true;
            })
            .catch((error) => {
                console.error('获取会话详情失败:', error);
                dispatch(
                    showNotification({
                        message: `获取聊天详情失败: ${error || '未知错误'}`,
                        type: 'danger',
                    })
                );
            })
            .finally(() => {
                setLoading(false);
            });

        // 组件卸载时清空消息
        return () => {
            // Don't reset messages when switching chats
            // dispatch(resetMessages());
            // 不要清空hasLoadedDetailRef，否则会导致重复加载
            // hasLoadedDetailRef.current = {}; // 清理ref缓存
        };
    }, [chatId, dispatch, createSessionStatus, createSessionId]);

    // 组件销毁时完全清空ref缓存
    useEffect(() => {
        return () => {
            hasLoadedDetailRef.current = {};
        };
    }, []);

    // 新会话自动添加欢迎消息
    useEffect(() => {
        // 如果是新聊天且没有任何消息，添加一条系统欢迎消息
        if (chatId && messages.length === 0 && !loading && messageStatus !== 'loading') {
            const selectedKb = knowledgeBase ||
                availableDatasets.find((ds) => ds.id === knowledgeBaseId) || { name: '知识库' };

            dispatch(
                addMessage({
                    id: 'welcome-' + Date.now(),
                    role: 'assistant',
                    content: `欢迎使用${selectedKb.name}。您可以向我提问任何相关问题。`,
                    created_at: new Date().toISOString(),
                })
            );
        }
    }, [chatId, messages.length, loading, messageStatus, knowledgeBase, knowledgeBaseId, availableDatasets, dispatch]);

    // 监听发送消息状态
    useEffect(() => {
        if (sendStatus === 'failed' && sendError) {
            dispatch(
                showNotification({
                    message: `发送失败: ${sendError}`,
                    type: 'danger',
                })
            );
            dispatch(resetSendMessageStatus());
        }
    }, [sendStatus, sendError, dispatch]);

    // 滚动到底部
    useEffect(() => {
        messagesEndRef.current?.scrollIntoView({ behavior: 'smooth' });
    }, [messages]);

    // 获取当前会话的知识库信息
    useEffect(() => {
        // 如果conversation有数据集信息，优先使用它
        if (conversation && conversation.datasets && conversation.datasets.length > 0) {
            return;
        }

        // 如果没有会话数据集信息，但有knowledgeBaseId，尝试从知识库列表中查找
        if (knowledgeBaseId && knowledgeBases.length === 0 && !availableDatasets.length) {
            dispatch(fetchAvailableDatasets());
        }
    }, [dispatch, knowledgeBaseId, knowledgeBases, conversation, availableDatasets]);

    const handleSendMessage = (e) => {
        e.preventDefault();

        if (!inputMessage.trim() || sendStatus === 'loading') return;

        console.log('准备发送消息:', inputMessage);
        console.log('当前会话ID:', chatId);

        // 获取知识库ID列表
        let dataset_id_list = [];

        // 优先使用组件状态中保存的知识库列表
        if (selectedKnowledgeBaseIds.length > 0) {
            // 使用已保存的知识库列表
            dataset_id_list = selectedKnowledgeBaseIds.map((id) => id.replace(/-/g, ''));
        } else if (conversation && conversation.datasets && conversation.datasets.length > 0) {
            // 如果已有会话，使用会话中的知识库
            dataset_id_list = conversation.datasets.map((ds) => ds.id.replace(/-/g, ''));
        } else if (knowledgeBaseId) {
            // 如果是新会话，使用当前选择的知识库
            // 可能是单个ID或以逗号分隔的多个ID
            const ids = knowledgeBaseId.split(',').map((id) => id.trim().replace(/-/g, ''));
            dataset_id_list = ids;
        } else if (availableDatasets.length > 0) {
            // 如果都没有，尝试使用可用知识库列表中的第一个
            dataset_id_list = [availableDatasets[0].id.replace(/-/g, '')];
        }

        if (dataset_id_list.length === 0) {
            dispatch(
                showNotification({
                    message: '发送失败：未选择知识库',
                    type: 'danger',
                })
            );
            return;
        }

        const requestBody = {
            dataset_id_list: dataset_id_list,
            question: inputMessage,
            conversation_id: chatId,
        };

        console.log('发送消息参数:', requestBody);

        // 创建WebSocket测试的AI回复消息ID - 必须先初始化
        const wsMessageId = `ws-${Date.now()}`;
        let wsMessageContent = '';

        // 添加用户消息
        const userMessageId = `user-${Date.now()}`;
        dispatch(
            addMessage({
                id: userMessageId,
                role: 'user',
                content: inputMessage,
                created_at: new Date().toISOString(),
            })
        );

        // 添加一个空白的临时消息用于WebSocket流式回复
        dispatch(
            addMessage({
                id: wsMessageId,
                role: 'assistant',
                content: '',
                created_at: new Date().toISOString(),
                is_streaming: true,
                is_websocket: true,
            })
        );

        // 获取聊天资源 - 在用户消息发送后立即请求资源
        dispatch(
            getChatResources({
                dataset_id_list,
                question: inputMessage,
                messageId: wsMessageId, // 将资源关联到助手消息
            })
        );

        sendChatMessageViaWebSocket(requestBody, (data) => {
            try {
                // 根据消息类型处理不同的响应
                // 处理部分内容
                if (data.message === 'partial') {
                    const newText = data.data.content || '';
                    wsMessageContent += newText;
                    wsMessageId = data.data.id;
                    // 更新消息内容 - 使用完全替换而不是追加，确保React检测到变化
                    dispatch(
                        updateMessage({
                            id: wsMessageId,
                            content: wsMessageContent,
                            is_streaming: true,
                            updated_at: new Date().toISOString(), // 添加时间戳强制更新
                        })
                    );

                    // 触发重新渲染
                    setTimeout(() => {
                        messagesEndRef.current?.scrollIntoView({ behavior: 'smooth' });
                    }, 50);
                }

                // 处理流结束
                if (data.message === '完成') {
                    console.log('WebSocket流式传输结束，最终内容:', data.data.content);

                    // 使用完整内容替换消息
                    dispatch(
                        updateMessage({
                            id: wsMessageId,
                            content: data.data.content || wsMessageContent,
                            is_streaming: false,
                            updated_at: new Date().toISOString(), // 添加时间戳强制更新
                        })
                    );

                    // 触发重新渲染和滚动
                    setTimeout(() => {
                        messagesEndRef.current?.scrollIntoView({ behavior: 'smooth' });
                    }, 50);
                }

                // 处理错误
                if (data.code !== 200 && data.code !== 201) {
                    console.error('WebSocket错误:', data.message);
                    dispatch(
                        showNotification({
                            message: `WebSocket错误: ${data.message}`,
                            type: 'warning',
                        })
                    );

                    // 更新消息显示错误
                    dispatch(
                        updateMessage({
                            id: wsMessageId,
                            content: `Error: ${data.message}`,
                            is_streaming: false,
                        })
                    );
                }
            } catch (error) {
                console.error('处理WebSocket响应失败:', error);
                // 更新消息显示错误
                dispatch(
                    updateMessage({
                        id: wsMessageId,
                        content: `处理WebSocket响应失败: ${error.message}`,
                        is_streaming: false,
                    })
                );
            }
        });

        // 清空输入框
        setInputMessage('');
    };

    // 渲染加载状态
    const renderLoading = () => (
        <div className='p-5 text-center'>
            <div className='spinner-border text-secondary' role='status'>
                <span className='visually-hidden'>加载中...</span>
            </div>
            <div className='mt-3 text-muted'>加载聊天记录...</div>
        </div>
    );

    // 渲染空消息状态
    const renderEmpty = () => {
        if (loading) return null;

        return (
            <div className='text-center my-5'>
                <p className='text-muted'>暂无消息，开始发送第一条消息吧</p>
            </div>
        );
    };

    return (
        <div className='chat-window d-flex flex-column h-100'>
            {/* Chat header */}
            <div className='p-3 border-bottom'>
                {conversation && conversation.datasets ? (
                    <>
                        <h5 className='mb-0'>{conversation.title}</h5>
                        {conversation.datasets.length > 0 && conversation.datasets[0].type && (
                            <small className='text-muted'>类型: {conversation.datasets[0].type}</small>
                        )}
                    </>
                ) : (
                    <h5 className='mb-0'>{loading || availableDatasetsLoading ? '加载中...' : '聊天'}</h5>
                )}
            </div>

            {/* Chat messages */}
            <div className='flex-grow-1 p-3 overflow-auto'>
                <div className='container'>
                    {messageStatus === 'loading'
                        ? renderLoading()
                        : messages.length === 0
                        ? renderEmpty()
                        : messages.map((message) => (
                              <div
                                  key={message.id}
                                  className={`d-flex ${
                                      message.role === 'user' ? 'align-items-end' : 'align-items-start'
                                  } mb-3 flex-column`}
                              >
                                  <div
                                      className={`chat-message p-3 rounded-3 ${
                                          message.role === 'user' ? 'bg-dark text-white' : 'bg-light'
                                      }`}
                                      style={{
                                          maxWidth: '75%',
                                          position: 'relative',
                                      }}
                                  >
                                      <div className='message-content'>
                                          {message.role === 'user' ? (
                                              message.content
                                          ) : (
                                              <SafeMarkdown
                                                  content={message.content}
                                                  isStreaming={message.is_streaming}
                                              />
                                          )}
                                          {message.is_streaming && (
                                              <span className='streaming-indicator'>
                                                  <span className='dot dot1'></span>
                                                  <span className='dot dot2'></span>
                                                  <span className='dot dot3'></span>
                                              </span>
                                          )}

                                          {/* 只在AI回复消息下方显示资源列表，仅当有内容时显示 */}
                                          {message.role === 'assistant' &&
                                              !message.is_streaming &&
                                              resources.messageId === message.id &&
                                              resources.status === 'succeeded' &&
                                              resources.items.length > 0 && (
                                                  <ResourceList resources={resources.items} />
                                              )}
                                      </div>
                                  </div>
                                  <div className='message-time small text-muted mt-1'>
                                      {message.created_at &&
                                          (() => {
                                              const messageDate = new Date(message.created_at);
                                              const today = new Date();

                                              // 检查是否是今天
                                              const isToday =
                                                  messageDate.getDate() === today.getDate() &&
                                                  messageDate.getMonth() === today.getMonth() &&
                                                  messageDate.getFullYear() === today.getFullYear();

                                              // 如果是今天，只显示时间；否则显示年月日和时间
                                              if (isToday) {
                                                  return messageDate.toLocaleTimeString('zh-CN', {
                                                      hour: '2-digit',
                                                      minute: '2-digit',
                                                  });
                                              } else {
                                                  return messageDate.toLocaleString('zh-CN', {
                                                      year: 'numeric',
                                                      month: '2-digit',
                                                      day: '2-digit',
                                                      hour: '2-digit',
                                                      minute: '2-digit',
                                                  });
                                              }
                                          })()}
                                      {message.is_streaming && ' · 正在生成...'}
                                  </div>
                              </div>
                          ))}

                    <div ref={messagesEndRef} />
                </div>
            </div>

            {/* Chat input */}
            <div className='p-3 border-top'>
                <form onSubmit={handleSendMessage} className='d-flex gap-2'>
                    <input
                        type='text'
                        className='form-control'
                        placeholder='输入你的问题...'
                        value={inputMessage}
                        onChange={(e) => setInputMessage(e.target.value)}
                        disabled={sendStatus === 'loading'}
                    />
                    <button
                        type='submit'
                        className='btn btn-dark d-flex align-items-center justify-content-center gap-2'
                        disabled={sendStatus === 'loading' || !inputMessage.trim()}
                    >
                        <SvgIcon className='send' color='#ffffff' />
                        <span className='ms-1' style={{ minWidth: 'fit-content' }}>
                            发送
                        </span>
                    </button>
                </form>
            </div>
        </div>
    );
}
