import React, { useRef, useState, useEffect } from 'react';
import { useDispatch } from 'react-redux';
import { uploadDocument, getKnowledgeBaseDocuments } from '../../../../store/knowledgeBase/knowledgeBase.thunks';
import { setIsUploading } from '../../../../store/upload/upload.slice';

/**
 * 文件上传模态框组件
 */
const FileUploadModal = ({ show, knowledgeBaseId, onClose, onMinimizeChange, isMinimized: externalIsMinimized }) => {
    const dispatch = useDispatch();
    const fileInputRef = useRef(null);
    const modalRef = useRef(null);
    const [selectedFiles, setSelectedFiles] = useState([]);
    const [isUploading, setIsUploading] = useState(false);
    const [fileError, setFileError] = useState('');
    const [uploadResults, setUploadResults] = useState(null);
    const [isMinimized, setIsMinimized] = useState(externalIsMinimized || false);

    // 同步外部的最小化状态
    useEffect(() => {
        if (externalIsMinimized !== undefined && externalIsMinimized !== isMinimized) {
            setIsMinimized(externalIsMinimized);
        }
    }, [externalIsMinimized]);

    // 处理上传区域点击事件
    const handleUploadAreaClick = () => {
        fileInputRef.current?.click();
    };

    // 处理拖拽事件
    const handleDragOver = (e) => {
        e.preventDefault();
        e.stopPropagation();
    };

    const handleDrop = (e) => {
        e.preventDefault();
        e.stopPropagation();

        if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
            handleFilesSelected(e.dataTransfer.files);
        }
    };

    const handleFileChange = (e) => {
        if (e.target.files && e.target.files.length > 0) {
            handleFilesSelected(e.target.files);
        }
    };

    const handleFilesSelected = (files) => {
        setFileError('');
        // 将FileList转为数组
        const filesArray = Array.from(files);
        setSelectedFiles((prev) => [...prev, ...filesArray]);
    };

    const removeFile = (index) => {
        setSelectedFiles((prev) => prev.filter((_, i) => i !== index));
    };

    const resetFileInput = () => {
        setSelectedFiles([]);
        setFileError('');
        setUploadResults(null);
        if (fileInputRef.current) {
            fileInputRef.current.value = '';
        }
    };

    const handleUpload = async () => {
        if (selectedFiles.length === 0) {
            setFileError('请至少选择一个要上传的文件');
            return;
        }

        setIsUploading(true);
        // Update Redux state
        dispatch({ type: 'upload/setIsUploading', payload: true });
        setUploadResults(null);

        try {
            // Upload the files
            const result = await dispatch(
                uploadDocument({
                    knowledge_base_id: knowledgeBaseId,
                    files: selectedFiles,
                })
            ).unwrap();

            // Refresh the documents list
            dispatch(
                getKnowledgeBaseDocuments({
                    knowledge_base_id: knowledgeBaseId,
                })
            );

            // Update UI with results
            setUploadResults(result);

            // Close dialog after success
            if (result.failed_count === 0) {
                setTimeout(() => {
                    handleClose();
                }, 3000);
            }
        } catch (error) {
            console.error('Upload failed:', error);
            setFileError('文件上传失败: ' + (error?.message || '未知错误'));
        } finally {
            setIsUploading(false);
            // Reset Redux state
            dispatch({ type: 'upload/setIsUploading', payload: false });
        }
    };

    const handleClose = () => {
        // 只有在非上传状态才允许关闭
        if (!isUploading) {
            resetFileInput();
            setIsMinimized(false); // 重置最小化状态
            onClose();
        } else {
            // 如果正在上传，提示用户
            alert('正在上传文件，请等待上传完成后再关闭。');
        }
    };

    // 处理最小化/最大化切换
    const toggleMinimize = () => {
        const newMinimizedState = !isMinimized;
        setIsMinimized(newMinimizedState);

        // 通知父组件最小化状态变化
        if (onMinimizeChange) {
            onMinimizeChange(newMinimizedState);
        }
    };

    // 处理显示状态变化
    useEffect(() => {
        // 当弹窗被显示时，如果之前是最小化状态，则恢复
        if (show && isMinimized) {
            setIsMinimized(false);
            if (onMinimizeChange) {
                onMinimizeChange(false);
            }
        }
    }, [show]);

    // 添加离开页面的警告
    useEffect(() => {
        const handleBeforeUnload = (e) => {
            if (isUploading) {
                // 标准方式阻止页面关闭
                e.preventDefault();
                // Chrome需要这个
                e.returnValue = '正在上传文件，离开页面可能会中断上传。确定要离开吗？';
                return e.returnValue;
            }
        };

        if (isUploading) {
            window.addEventListener('beforeunload', handleBeforeUnload);
        }

        return () => {
            window.removeEventListener('beforeunload', handleBeforeUnload);
        };
    }, [isUploading]);

    // 如果不显示且不是最小化状态，则不渲染任何内容
    if (!show && !isMinimized) return null;

    // 最小化模式的渲染
    if (isMinimized) {
        return (
            <div
                className='minimized-upload-modal bg-white rounded shadow d-flex align-items-center'
                style={{
                    position: 'fixed',
                    bottom: '20px',
                    right: '20px',
                    padding: '10px 15px',
                    zIndex: 1050,
                    border: '1px solid #ddd',
                    cursor: 'pointer',
                }}
                onClick={toggleMinimize}
            >
                <div className='d-flex align-items-center'>
                    {isUploading ? (
                        <span className='spinner-border spinner-border-sm me-2' role='status' aria-hidden='true'></span>
                    ) : (
                        <span className='me-2'>📁</span>
                    )}
                    <span>
                        {isUploading
                            ? `正在上传文件 (${selectedFiles.length})...`
                            : `文件上传 (${selectedFiles.length})`}
                    </span>
                </div>
            </div>
        );
    }

    // 只有在显示状态下渲染完整弹窗
    if (!show) return null;

    // 完整模式的渲染
    return (
        <div
            ref={modalRef}
            className='modal-backdrop'
            style={{
                position: 'fixed',
                top: 0,
                left: 0,
                right: 0,
                bottom: 0,
                backgroundColor: 'rgba(0, 0, 0, 0.5)',
                display: 'flex',
                alignItems: 'center',
                justifyContent: 'center',
                zIndex: 1050,
            }}
        >
            <div
                className='modal-content bg-white rounded shadow'
                style={{
                    width: '600px',
                    maxWidth: '90%',
                    padding: '20px',
                    maxHeight: '90vh',
                    overflowY: 'auto',
                }}
            >
                <div className='modal-header d-flex justify-content-between align-items-center mb-3'>
                    <h5 className='modal-title m-0'>上传文档</h5>
                    <div className='d-flex align-items-center' style={{ gap: '8px' }}>
                        {/* Minimize button */}
                        <button
                            type='button'
                            onClick={toggleMinimize}
                            aria-label='Minimize'
                            className='btn p-0 d-flex align-items-center justify-content-center'
                            style={{
                                width: '22px',
                                height: '22px',
                                border: 'none',
                                background: 'transparent',
                                opacity: 0.5,
                            }}
                            onMouseOver={(e) => (e.currentTarget.style.opacity = '0.75')}
                            onMouseOut={(e) => (e.currentTarget.style.opacity = '0.5')}
                        >
                            <svg
                                xmlns='http://www.w3.org/2000/svg'
                                viewBox='0 0 512 512'
                                width='14'
                                height='14'
                                fill='currentColor'
                            >
                                <path d='M32 416c-17.7 0-32 14.3-32 32s14.3 32 32 32l448 0c17.7 0 32-14.3 32-32s-14.3-32-32-32L32 416z' />
                            </svg>
                        </button>

                        {/* Close button */}
                        <button
                            type='button'
                            className='btn-close'
                            onClick={handleClose}
                            disabled={isUploading}
                            aria-label='Close'
                            style={{
                                width: '22px',
                                height: '22px',
                                padding: 0,
                            }}
                        ></button>
                    </div>
                </div>
                <div className='modal-body'>
                    <div
                        className={`mb-3 p-4 border rounded text-center ${
                            fileError ? 'border-danger' : 'border-dashed'
                        }`}
                        style={{ cursor: isUploading ? 'not-allowed' : 'pointer' }}
                        onClick={!isUploading ? handleUploadAreaClick : undefined}
                        onDrop={!isUploading ? handleDrop : undefined}
                        onDragOver={handleDragOver}
                    >
                        <input
                            type='file'
                            ref={fileInputRef}
                            className='d-none'
                            onChange={handleFileChange}
                            accept='.pdf,.doc,.docx,.txt,.md,.csv,.xlsx,.xls'
                            disabled={isUploading}
                            multiple
                        />
                        <div>
                            <p className='mb-1'>点击或拖拽文件到此处上传</p>
                            <p className='text-muted small mb-0'>支持 PDF, Word, Excel, TXT, Markdown, CSV 等格式</p>
                        </div>
                        {fileError && <div className='text-danger mt-2'>{fileError}</div>}
                    </div>

                    {/* 选择的文件列表 */}
                    {selectedFiles.length > 0 && (
                        <div className='mb-3'>
                            <p className='fw-bold mb-2'>已选择 {selectedFiles.length} 个文件：</p>
                            <ul className='list-group'>
                                {selectedFiles.map((file, index) => (
                                    <li
                                        key={index}
                                        className='list-group-item d-flex justify-content-between align-items-center'
                                    >
                                        <div
                                            className='d-flex align-items-center'
                                            style={{ maxWidth: 'calc(100% - 70px)' }}
                                        >
                                            <div className='text-truncate'>{file.name}</div>
                                            <div className='text-nowrap ms-2 text-muted' style={{ flexShrink: 0 }}>
                                                ({(file.size / 1024).toFixed(0)} KB)
                                            </div>
                                        </div>
                                        {!isUploading && (
                                            <button
                                                className='btn btn-sm btn-outline-danger'
                                                onClick={() => removeFile(index)}
                                                style={{ minWidth: '60px' }}
                                            >
                                                移除
                                            </button>
                                        )}
                                    </li>
                                ))}
                            </ul>
                        </div>
                    )}

                    {/* 上传结果显示 */}
                    {uploadResults && (
                        <div className='mt-3 border rounded p-3'>
                            <h6 className='mb-2'>上传结果</h6>
                            <p className='mb-2'>
                                总文件: {uploadResults.total_files}, 成功: {uploadResults.uploaded_count}, 失败:{' '}
                                {uploadResults.failed_count}
                            </p>

                            {uploadResults.documents && uploadResults.documents.length > 0 && (
                                <>
                                    <p className='mb-1 fw-bold text-success'>上传成功:</p>
                                    <ul className='list-group mb-2'>
                                        {uploadResults.documents.map((doc) => (
                                            <li key={doc.id} className='list-group-item py-2 d-flex align-items-center'>
                                                <span className='badge bg-success me-2'>✓</span>
                                                <span className='text-truncate'>{doc.name}</span>
                                            </li>
                                        ))}
                                    </ul>
                                </>
                            )}

                            {uploadResults.failed_documents && uploadResults.failed_documents.length > 0 && (
                                <>
                                    <p className='mb-1 fw-bold text-danger'>上传失败:</p>
                                    <ul className='list-group'>
                                        {uploadResults.failed_documents.map((doc, index) => (
                                            <li key={index} className='list-group-item py-2 d-flex align-items-center'>
                                                <span className='badge bg-danger me-2'>✗</span>
                                                <div className='text-truncate'>
                                                    {doc.name}
                                                    {doc.reason && (
                                                        <small className='ms-2 text-danger'>({doc.reason})</small>
                                                    )}
                                                </div>
                                            </li>
                                        ))}
                                    </ul>
                                </>
                            )}
                        </div>
                    )}
                </div>
                <div className='modal-footer gap-2'>
                    <button type='button' className='btn btn-secondary' onClick={handleClose} disabled={isUploading}>
                        关闭
                    </button>
                    <button
                        type='button'
                        className='btn btn-dark'
                        onClick={handleUpload}
                        disabled={selectedFiles.length === 0 || isUploading}
                    >
                        {isUploading ? (
                            <>
                                <span
                                    className='spinner-border spinner-border-sm me-2'
                                    role='status'
                                    aria-hidden='true'
                                ></span>
                                上传中...
                            </>
                        ) : (
                            `上传文档${selectedFiles.length > 0 ? ` (${selectedFiles.length})` : ''}`
                        )}
                    </button>
                </div>
            </div>
        </div>
    );
};

export default FileUploadModal;
