import axios from 'axios';
import CryptoJS from 'crypto-js';
import { mockGet, mockPost, mockPut, mockDelete } from './mockApi';

const secretKey = import.meta.env.VITE_SECRETKEY;

// API连接状态
let isServerDown = false;
let hasCheckedServer = false;

// Create Axios instance with base URL
const api = axios.create({
    baseURL: '/api',
    withCredentials: true, // Include cookies if needed
});

// Request Interceptor
api.interceptors.request.use(
    (config) => {
        const encryptedToken = sessionStorage.getItem('token') || '';
        if (encryptedToken) {
            const decryptedToken = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
            config.headers.Authorization = `Token ${decryptedToken}`;
        }
        return config;
    },
    (error) => {
        console.error('Request error:', error);
        return Promise.reject(error);
    }
);

// Response Interceptor
api.interceptors.response.use(
    (response) => {
        // 如果成功收到响应，表示服务器正常工作
        if (!hasCheckedServer) {
            console.log('Server is up and running');
            isServerDown = false;
            hasCheckedServer = true;
        }
        return response;
    },
    (error) => {
        // 处理服务器无法连接的情况
        if (!error.response || error.code === 'ECONNABORTED' || error.message.includes('Network Error')) {
            console.error('Server appears to be down. Switching to mock data.');
            isServerDown = true;
            hasCheckedServer = true;
        }

        // Handle errors in the response
        if (error.response) {
            // monitor /verify
            if (error.response.status === 401 && error.config.url === '/auth/verify-token/') {
                if (window.location.pathname !== '/login' && window.location.pathname !== '/signup') {
                    window.location.href = '/login';
                }
            }

            // The request was made and the server responded with a status code
            console.error('API Error Response:', error.response.status, error.response.data.message);
            // alert(`Error: ${error.response.status} - ${error.response.data.message || 'Something went wrong'}`);
        } else if (error.request) {
            // The request was made but no response was received
            console.error('API Error Request:', error.request);
            // alert('Network error: No response from server');
        } else {
            // Something happened in setting up the request
            console.error('API Error Message:', error.message);
            // alert('Error: ' + error.message);
        }
        return Promise.reject(error); // Reject the promise
    }
);

// 检查服务器状态
export const checkServerStatus = async () => {
    try {
        // await api.get('/health-check', { timeout: 3000 });
        isServerDown = false;
        hasCheckedServer = true;
        console.log('Server connection established');
        return true;
    } catch (error) {
        isServerDown = true;
        hasCheckedServer = true;
        console.error('Server connection failed, using mock data');
        return false;
    }
};

// 初始检查服务器状态
checkServerStatus();

// Define common HTTP methods with fallback to mock API
const get = async (url, params = {}) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] GET ${url}`);
            return await mockGet(url, params);
        }

        const res = await api.get(url, { ...params });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for GET ${url}`);
            return await mockGet(url, params);
        }
        throw error;
    }
};

// Handle POST requests for JSON data with fallback to mock API
const post = async (url, data, isMultipart = false) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] POST ${url}`);
            return await mockPost(url, data);
        }

        const headers = isMultipart
            ? { 'Content-Type': 'multipart/form-data' } // For file uploads
            : { 'Content-Type': 'application/json' }; // For JSON data

        const res = await api.post(url, data, { headers });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for POST ${url}`);
            return await mockPost(url, data);
        }
        throw error;
    }
};

// Handle PUT requests with fallback to mock API
const put = async (url, data) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] PUT ${url}`);
            return await mockPut(url, data);
        }

        const res = await api.put(url, data, {
            headers: { 'Content-Type': 'application/json' },
        });
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for PUT ${url}`);
            return await mockPut(url, data);
        }
        throw error;
    }
};

// Handle DELETE requests with fallback to mock API
const del = async (url) => {
    try {
        const res = await api.delete(url);
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for DELETE ${url}`);
            return await mockDelete(url);
        }
        throw error;
    }
};

const upload = async (url, data) => {
    try {
        if (isServerDown) {
            console.log(`[MOCK MODE] Upload ${url}`);
            return await mockPost(url, data, true);
        }

        const axiosInstance = await axios.create({
            baseURL: '/api',
            headers: {
                'Content-Type': 'multipart/form-data',
            },
        });
        const res = await axiosInstance.post(url, data);
        return res.data;
    } catch (error) {
        if (!hasCheckedServer || (error.request && !error.response)) {
            console.log(`Failed to connect to server. Falling back to mock API for Upload ${url}`);
            return await mockPost(url, data, true);
        }
        throw error;
    }
};

// 手动切换到模拟API（为调试目的）
export const switchToMockApi = () => {
    isServerDown = true;
    hasCheckedServer = true;
    console.log('Manually switched to mock API');
};

// 手动切换回真实API
export const switchToRealApi = async () => {
    // 重新检查服务器状态
    const isServerUp = await checkServerStatus();
    console.log(isServerUp ? 'Switched back to real API' : 'Server still down, continuing with mock API');
    return isServerUp;
};

// Handle streaming requests
const streamRequest = async (url, data, onChunk, onError) => {
    try {
        // 获取认证Token
        const encryptedToken = sessionStorage.getItem('token') || '';
        let token = '';
        if (encryptedToken) {
            token = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
        }

        // 使用fetch API进行流式请求
        const response = await fetch(`/api${url}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
            },
            body: JSON.stringify(data),
        });

        if (!response.ok) {
            throw new Error(`HTTP error! Status: ${response.status}`);
        }

        // 检查是否为SSE (Server-Sent Events)格式
        const contentType = response.headers.get('Content-Type');
        const isSSE = contentType && contentType.includes('text/event-stream');
        console.log('响应内容类型:', contentType, '是否SSE:', isSSE);

        // 处理SSE格式
        if (isSSE) {
            const reader = response.body.getReader();
            const decoder = new TextDecoder();
            let buffer = '';
            let conversationId = null;

            // 处理流式数据
            while (true) {
                const { done, value } = await reader.read();
                if (done) break;

                // 解码并处理数据 - 不使用stream选项以确保完整解码
                const chunk = decoder.decode(value, { stream: false });
                buffer += chunk;

                // 按行分割并处理JSON
                const lines = buffer.split('\n');
                buffer = lines.pop() || ''; // 保留最后一行（可能不完整）

                for (const line of lines) {
                    if (!line.trim()) continue;

                    try {
                        // 提取data:后面的JSON部分
                        let jsonStr = line;
                        if (line.startsWith('data:')) {
                            jsonStr = line.substring(5).trim();
                            console.log('SSE数据块:', jsonStr);
                        }

                        // 尝试解析JSON
                        const parsedData = JSON.parse(jsonStr);
                        if (parsedData.code === 200 && parsedData.data && parsedData.data.conversation_id) {
                            conversationId = parsedData.data.conversation_id;
                        }

                        // 立即调用处理函数
                        onChunk(jsonStr);
                    } catch (e) {
                        console.warn('解析JSON失败:', line, e);
                    }
                }
            }

            return { success: true, conversation_id: conversationId };
        }
        // 处理常规JSON响应
        else {
            // 原始响应可能是单个JSON对象而不是流
            const responseData = await response.json();
            console.log('接收到非流式响应:', responseData);

            if (responseData.code === 200) {
                // 模拟分段处理
                const content = responseData.data?.content || '';
                const conversationId = responseData.data?.conversation_id;

                // 每100个字符分段处理
                let offset = 0;
                const chunkSize = 100;

                while (offset < content.length) {
                    const isLast = offset + chunkSize >= content.length;
                    const chunk = content.substring(offset, offset + chunkSize);

                    // 构造类似流式传输的JSON
                    const chunkData = {
                        code: 200,
                        message: 'partial',
                        data: {
                            content: chunk,
                            conversation_id: conversationId,
                            is_end: isLast,
                        },
                    };

                    // 调用处理函数
                    onChunk(JSON.stringify(chunkData));

                    // 暂停一下让UI有时间更新
                    await new Promise((resolve) => setTimeout(resolve, 50));

                    offset += chunkSize;
                }

                return { success: true, conversation_id: conversationId };
            }

            // 如果不是成功响应，直接传递原始数据
            onChunk(JSON.stringify(responseData));
            return { success: responseData.code === 200, conversation_id: responseData.data?.conversation_id };
        }
    } catch (error) {
        console.error('Streaming request failed:', error);
        if (onError) {
            onError(error);
        }
        throw error;
    }
};

// 权限相关API
export const applyPermission = (data) => {
    return post('/permissions/', data);
};

export const updatePermission = (data) => {
    return post('/permissions/update_permission/', data);
};

export const approvePermission = (permissionId) => {
    return post(`/permissions/approve_permission/${permissionId}`);
};

export const rejectPermission = (permissionId) => {
    return post(`/permissions/reject_permission/${permissionId}`);
};

/**
 * 获取聊天回复相关资源列表
 * @param {Object} data - 请求参数
 * @param {Array} data.dataset_id_list - 知识库ID列表
 * @param {string} data.question - 用户问题
 * @returns {Promise<Object>} API响应
 */
export const fetchChatResources = async (data) => {
    try {
        const response = await post('/chat-history/hit_test/', data);
        return response;
    } catch (error) {
        console.error('获取聊天资源失败:', error);
        throw error;
    }
};

export { get, post, put, del, upload, streamRequest };
