import {
    addNotification,
    markNotificationAsRead,
    setWebSocketConnected,
} from '../store/notificationCenter/notificationCenter.slice';
import store from '../store/store'; // 修改为默认导出
import CryptoJS from 'crypto-js';

const secretKey = import.meta.env.VITE_SECRETKEY;

// 从环境变量获取 API URL
const API_URL = import.meta.env.VITE_API_URL || 'http://81.69.223.133:8008';
// 将 HTTP URL 转换为 WebSocket URL
const WS_BASE_URL = API_URL.replace(/^http/, 'ws').replace(/\/api\/?$/, '');
const WS_CHAT_URL = 'ws://81.69.223.133:8008/ws/chat/stream/';

let socket = null;
let chatSocket = null;
let reconnectTimer = null;
let pingInterval = null;
let reconnectAttempts = 0; // 添加重连尝试计数器
let globalReconnectAttempts = 0; // 添加全局重连计数器
const RECONNECT_DELAY = 5000; // 5秒后尝试重连
const PING_INTERVAL = 30000; // 30秒发送一次ping
const MAX_RECONNECT_ATTEMPTS = 3; // 最大重连尝试次数
const MAX_GLOBAL_RECONNECT_ATTEMPTS = 3; // 单个会话中允许的总重连次数

// 添加的聊天消息回调处理
let chatMessageCallback = null;

/**
 * 初始化WebSocket连接
 * @returns {Promise<WebSocket>} WebSocket连接实例
 */
export const initWebSocket = () => {
    return new Promise((resolve, reject) => {
        // 检查全局重连次数
        if (globalReconnectAttempts >= MAX_GLOBAL_RECONNECT_ATTEMPTS) {
            console.warn(`已达到全局最大重连次数(${MAX_GLOBAL_RECONNECT_ATTEMPTS})，不再尝试重连`);
            reject(new Error('Maximum global reconnection attempts reached'));
            return;
        }

        // 如果已经有一个连接，先关闭它
        if (socket && socket.readyState !== WebSocket.CLOSED) {
            console.log('关闭已有WebSocket连接');
            socket.close(1000, 'Normal closure, reconnecting');
        }

        // 清除之前的定时器
        if (reconnectTimer) {
            clearTimeout(reconnectTimer);
            reconnectTimer = null;
        }

        if (pingInterval) {
            clearInterval(pingInterval);
            pingInterval = null;
        }

        try {
            // 从sessionStorage获取token
            const encryptedToken = sessionStorage.getItem('token');
            if (!encryptedToken) {
                console.error('No token found, cannot connect to notification service');
                store.dispatch(setWebSocketConnected(false));
                reject(new Error('No token found'));
                return;
            }

            let token = '';
            try {
                token = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
                if (!token) {
                    throw new Error('Token decryption resulted in empty string');
                }
            } catch (e) {
                console.error('Failed to decrypt token:', e);
                store.dispatch(setWebSocketConnected(false));
                reject(new Error('Invalid token'));
                return;
            }

            const wsUrl = `${WS_BASE_URL}/ws/notifications?token=${token}`;
            console.log('正在连接WebSocket...', wsUrl.substring(0, wsUrl.indexOf('?')));

            socket = new WebSocket(wsUrl);

            // 连接建立时的处理
            socket.onopen = () => {
                console.log('WebSocket 连接成功!');
                reconnectAttempts = 0; // 连接成功后重置当前重连计数器
                // 不重置全局重连计数器，确保总重连次数限制

                // 更新Redux中的连接状态
                store.dispatch(setWebSocketConnected(true));

                // 订阅通知频道
                subscribeToNotifications();

                // 设置定时发送ping消息
                if (pingInterval) clearInterval(pingInterval);
                pingInterval = setInterval(() => {
                    if (socket && socket.readyState === WebSocket.OPEN) {
                        sendPing();
                    }
                }, PING_INTERVAL);

                resolve(socket);
            };

            // 接收消息的处理
            socket.onmessage = (event) => {
                try {
                    const data = JSON.parse(event.data);
                    handleWebSocketMessage(data);
                } catch (error) {
                    console.error('解析WebSocket消息失败:', error, 'Raw message:', event.data);
                }
            };

            // 错误处理
            socket.onerror = (error) => {
                console.error('WebSocket连接错误:', error);
                // 不立即更新Redux状态，让onclose处理
            };

            // 连接关闭时的处理
            socket.onclose = (event) => {
                console.log(
                    `WebSocket连接关闭: 代码=${event.code} 原因="${event.reason || '未知'}" 是否干净=${event.wasClean}`
                );

                // 清除ping定时器
                if (pingInterval) {
                    clearInterval(pingInterval);
                    pingInterval = null;
                }

                // 更新Redux中的连接状态
                store.dispatch(setWebSocketConnected(false));

                // 如果不是正常关闭，尝试重连
                if (!event.wasClean && event.code !== 1000 && event.code !== 1001) {
                    // 检查是否已达到最大重连次数
                    if (
                        reconnectAttempts < MAX_RECONNECT_ATTEMPTS &&
                        globalReconnectAttempts < MAX_GLOBAL_RECONNECT_ATTEMPTS
                    ) {
                        reconnectAttempts++;
                        globalReconnectAttempts++;

                        const delay = Math.min(RECONNECT_DELAY * reconnectAttempts, 15000); // 指数退避，但最大15秒
                        console.log(
                            `WebSocket将在${
                                delay / 1000
                            }秒后尝试重连 (第${reconnectAttempts}/${MAX_RECONNECT_ATTEMPTS}次, 总计${globalReconnectAttempts}/${MAX_GLOBAL_RECONNECT_ATTEMPTS}次)`
                        );

                        reconnectTimer = setTimeout(() => {
                            console.log('正在尝试重新连接WebSocket...');
                            initWebSocket()
                                .then(() => {
                                    console.log('WebSocket重连成功');
                                })
                                .catch((err) => {
                                    console.error('WebSocket重连失败:', err);
                                    // 重连失败时更新Redux中的连接状态
                                    store.dispatch(setWebSocketConnected(false));
                                });
                        }, delay);
                    } else {
                        const msg = `已达到最大重连次数 (当前${reconnectAttempts}/${MAX_RECONNECT_ATTEMPTS}, 总计${globalReconnectAttempts}/${MAX_GLOBAL_RECONNECT_ATTEMPTS})`;
                        console.warn(msg);
                        // 达到最大重连次数时更新Redux中的连接状态
                        store.dispatch(setWebSocketConnected(false));
                    }
                } else {
                    console.log('正常关闭，不会尝试重连');
                }
            };
        } catch (error) {
            console.error('Error initializing WebSocket:', error);
            // 更新Redux中的连接状态
            store.dispatch(setWebSocketConnected(false));
            reject(error);
        }
    });
};

/**
 * 订阅通知频道
 */
export const subscribeToNotifications = () => {
    if (socket && socket.readyState === WebSocket.OPEN) {
        const subscribeMessage = {
            type: 'subscribe',
            channel: 'notifications',
        };
        socket.send(JSON.stringify(subscribeMessage));
    }
};

/**
 * 发送ping消息（保持连接活跃）
 */
export const sendPing = () => {
    if (socket && socket.readyState === WebSocket.OPEN) {
        try {
            const pingMessage = {
                type: 'ping',
                timestamp: new Date().toISOString(),
            };
            socket.send(JSON.stringify(pingMessage));
            console.debug('已发送 ping 消息');
        } catch (error) {
            console.error('发送 ping 消息失败:', error);
        }
    }
};

/**
 * 确认已读通知
 * @param {string} notificationId 通知ID
 */
export const acknowledgeNotification = (notificationId) => {
    if (socket && socket.readyState === WebSocket.OPEN) {
        const ackMessage = {
            type: 'acknowledge',
            notification_id: notificationId,
        };
        socket.send(JSON.stringify(ackMessage));

        // 使用 store.dispatch 替代 dispatch
        store.dispatch(markNotificationAsRead(notificationId));
    }
};

/**
 * 关闭WebSocket连接
 * @param {boolean} [permanent=false] 是否永久关闭不再重连
 */
export const closeWebSocket = (permanent = false) => {
    // 停止重连尝试
    if (reconnectTimer) {
        clearTimeout(reconnectTimer);
        reconnectTimer = null;
    }

    // 停止ping
    if (pingInterval) {
        clearInterval(pingInterval);
        pingInterval = null;
    }

    // 如果是永久关闭，重置重连计数器
    if (permanent) {
        globalReconnectAttempts = MAX_GLOBAL_RECONNECT_ATTEMPTS; // 设置为最大值阻止重连
    }

    // 关闭连接
    if (socket) {
        if (socket.readyState === WebSocket.OPEN || socket.readyState === WebSocket.CONNECTING) {
            console.log(`手动关闭WebSocket连接${permanent ? '(永久)' : ''}`);
            socket.close(1000, '用户主动关闭');
        }
        socket = null;
    }

    // 更新Redux中的连接状态
    store.dispatch(setWebSocketConnected(false));
};

/**
 * 重置WebSocket连接状态，允许重新尝试连接
 */
export const resetWebSocketState = () => {
    closeWebSocket(true); // 先关闭当前连接

    // 重置所有计数器和状态
    reconnectAttempts = 0;
    globalReconnectAttempts = 0;

    console.log('WebSocket连接状态已重置，可以重新尝试连接');
};

/**
 * 处理接收到的WebSocket消息
 * @param {Object} data 解析后的消息数据
 */
const handleWebSocketMessage = (data) => {
    if (!data || typeof data !== 'object') {
        console.warn('收到无效的WebSocket消息:', data);
        return;
    }

    try {
        switch (data.type) {
            case 'connection_established':
                console.log(`WebSocket连接已建立，用户ID: ${data.user_id}`);
                break;

            case 'notification':
                if (!data.data) {
                    console.warn('收到无效的通知数据:', data);
                    return;
                }
                console.log('收到新通知:', data.data.title);
                // 将通知添加到Redux store
                store.dispatch(addNotification(processNotification(data)));
                break;

            case 'pong':
                console.debug(`收到pong响应，时间戳: ${data.timestamp}`);
                break;

            case 'error':
                console.error(`WebSocket错误: ${data.code} - ${data.message}`);
                break;

            default:
                console.log('收到未知类型的消息:', data.type, data);
        }
    } catch (error) {
        console.error('处理WebSocket消息时发生错误:', error, 'Message:', data);
    }
};

/**
 * 处理通知数据，转换为应用内通知格式
 * @param {Object} data 通知数据
 * @returns {Object} 处理后的通知数据
 */
const processNotification = (data) => {
    const { data: notificationData } = data;

    let icon = 'bi-info-circle';
    if (notificationData.category === 'system') {
        icon = 'bi-info-circle';
    } else if (notificationData.category === 'permission') {
        icon = 'bi-shield';
    }

    // 计算时间显示
    const createdAt = new Date(notificationData.created_at);
    const now = new Date();
    const diffMs = now - createdAt;
    const diffMins = Math.floor(diffMs / 60000);
    const diffHours = Math.floor(diffMins / 60);
    const diffDays = Math.floor(diffHours / 24);

    let timeDisplay;
    if (diffMins < 60) {
        timeDisplay = `${diffMins}分钟前`;
    } else if (diffHours < 24) {
        timeDisplay = `${diffHours}小时前`;
    } else {
        timeDisplay = `${diffDays}天前`;
    }

    return {
        id: notificationData.id,
        type: notificationData.category,
        icon,
        title: notificationData.title,
        content: notificationData.content,
        time: timeDisplay,
        hasDetail: true,
        isRead: notificationData.is_read,
        created_at: notificationData.created_at,
        metadata: notificationData.metadata || {},
    };
};

/**
 * 初始化聊天WebSocket连接
 * @returns {Promise<WebSocket>} WebSocket连接实例
 */
export const initChatWebSocket = () => {
    return new Promise((resolve, reject) => {
        // 如果已经有一个连接，先关闭它
        if (chatSocket && chatSocket.readyState !== WebSocket.CLOSED) {
            console.log('关闭已有Chat WebSocket连接');
            chatSocket.close(1000, 'Normal closure, reconnecting');
        }

        try {
            // 从sessionStorage获取token
            const encryptedToken = sessionStorage.getItem('token');
            if (!encryptedToken) {
                console.error('No token found, cannot connect to chat service');
                reject(new Error('No token found'));
                return;
            }

            let token = '';
            try {
                token = CryptoJS.AES.decrypt(encryptedToken, secretKey).toString(CryptoJS.enc.Utf8);
                if (!token) {
                    throw new Error('Token decryption resulted in empty string');
                }
            } catch (e) {
                console.error('Failed to decrypt token:', e);
                reject(new Error('Invalid token'));
                return;
            }

            // 构建WebSocket URL
            const wsUrl = `${WS_CHAT_URL}?token=${token}`;
            console.log('正在连接Chat WebSocket...', wsUrl.substring(0, wsUrl.indexOf('?')));

            // 创建WebSocket连接
            chatSocket = new WebSocket(wsUrl);

            // 设置超时
            const connectionTimeout = setTimeout(() => {
                if (chatSocket.readyState !== WebSocket.OPEN) {
                    console.error('Chat WebSocket连接超时');
                    chatSocket.close();
                    reject(new Error('Connection timeout'));
                }
            }, 10000); // 10秒超时

            // 连接建立时的处理
            chatSocket.onopen = () => {
                console.log('Chat WebSocket 连接成功!');
                clearTimeout(connectionTimeout);
                resolve(chatSocket);
            };

            // 接收消息的处理
            chatSocket.onmessage = (event) => {
                try {
                    // 解析消息并传递给回调
                    const message = JSON.parse(event.data);

                    if (chatMessageCallback) {
                        chatMessageCallback(message);
                    }
                } catch (error) {
                    console.error('处理Chat WebSocket消息失败:', error, 'Raw message:', event.data);
                }
            };

            // 错误处理
            chatSocket.onerror = (error) => {
                console.error('Chat WebSocket连接错误:', error);
                clearTimeout(connectionTimeout);
                reject(error);
            };

            // 连接关闭时的处理
            chatSocket.onclose = (event) => {
                console.log(
                    `Chat WebSocket连接关闭: 代码=${event.code} 原因="${event.reason || '未知'}" 是否干净=${
                        event.wasClean
                    }`
                );
                clearTimeout(connectionTimeout);
            };
        } catch (error) {
            console.error('Error initializing Chat WebSocket:', error);
            reject(error);
        }
    });
};

/**
 * 通过WebSocket发送聊天消息
 * @param {Object} message 聊天消息
 * @param {Function} callback 接收消息的回调函数
 */
export const sendChatMessageViaWebSocket = (message, callback) => {
    // 保存回调函数
    chatMessageCallback = callback;

    // 确保WebSocket连接已建立
    if (!chatSocket || chatSocket.readyState !== WebSocket.OPEN) {
        console.log('WebSocket未连接，正在初始化连接...');
        return initChatWebSocket()
            .then(() => {
                console.log('WebSocket连接已建立，发送消息:', message);
                chatSocket.send(JSON.stringify(message));
            })
            .catch((error) => {
                console.error('Chat WebSocket连接失败:', error);
                // 如果连接失败，调用回调传递错误
                if (callback) {
                    callback({
                        code: 500,
                        message: `WebSocket连接失败: ${error.message}`,
                        data: {
                            content: `连接失败: ${error.message}`,
                            is_end: true,
                        },
                    });
                }
            });
    } else {
        // 如果连接已建立，直接发送消息
        console.log('WebSocket已连接，直接发送消息:', message);
        chatSocket.send(JSON.stringify(message));
    }
};

/**
 * 处理聊天WebSocket消息
 * 流式内容结构类似于:
 * {
 *   "code": 200,
 *   "message": "开始流式传输" | "partial" | "完成",
 *   "data": {
 *      "content": "消息内容",
 *      "is_end": true/false
 *   }
 * }
 * @param {string} data 原始消息数据
 * @param {Function} callback 处理消息的回调函数
 */
export const processChatWebSocketMessage = (data, callback) => {
    try {
        const message = JSON.parse(data);

        // 调用回调处理消息
        if (callback) {
            callback(message);
        }

        return message;
    } catch (error) {
        console.error('解析Chat WebSocket消息失败:', error);
    }
};

/**
 * 关闭聊天WebSocket连接
 */
export const closeChatWebSocket = () => {
    // 清除回调
    chatMessageCallback = null;

    // 关闭连接
    if (chatSocket) {
        if (chatSocket.readyState === WebSocket.OPEN || chatSocket.readyState === WebSocket.CONNECTING) {
            console.log('手动关闭Chat WebSocket连接');
            chatSocket.close(1000, '用户主动关闭');
        }
        chatSocket = null;
    }

    console.log('Chat WebSocket连接已关闭');
};
