import { createSlice } from '@reduxjs/toolkit';
import {
    fetchAvailableDatasets,
    fetchChats,
    createChat,
    updateChat,
    deleteChat,
    createChatRecord,
    fetchConversationDetail,
    createConversation,
} from './chat.thunks';
import { sendMessage } from './chat.messages.thunks';

// 初始状态
const initialState = {
    // Chat session creation state
    createSession: {
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
        sessionId: null,
    },
    // Send message state
    sendMessage: {
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
    },
    // 可用于聊天的知识库列表
    availableDatasets: {
        items: [],
        status: 'idle',
        error: null,
    },
    // 操作状态（创建、更新、删除）
    operations: {
        status: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        error: null,
    },
    // 聊天列表状态
    list: {
        items: [],
        total: 0,
        status: 'idle',
        error: null,
        messageStatus: 'idle', // 'idle' | 'loading' | 'succeeded' | 'failed'
        messageError: null,
    },
    // 当前聊天
    currentChat: {
        data: null,
        status: 'idle',
        error: null,
        conversationId: null, // 当前选择的会话ID
    },
    // 聊天资源引用
    resources: {
        messageId: null,
        items: [],
        status: 'idle',
        error: null,
    },
};

// 创建 slice
const chatSlice = createSlice({
    name: 'chat',
    initialState,
    reducers: {
        // 重置操作状态
        resetOperationStatus: (state) => {
            state.operations.status = 'idle';
            state.operations.error = null;
        },

        // 重置当前聊天
        resetCurrentChat: (state) => {
            state.currentChat.data = null;
            state.currentChat.status = 'idle';
            state.currentChat.error = null;
            state.currentChat.conversationId = null;
        },

        // 设置当前聊天
        setCurrentChat: (state, action) => {
            state.currentChat.data = action.payload;
            state.currentChat.status = 'succeeded';
            if (action.payload && action.payload.conversation_id) {
                state.currentChat.conversationId = action.payload.conversation_id;
            }
        },

        // 重置消息状态
        resetMessages: (state) => {
            if (state.currentChat.conversationId) {
                const chatIndex = state.list.items.findIndex(
                    (chat) => chat.conversation_id === state.currentChat.conversationId
                );
                if (chatIndex !== -1) {
                    // 只重置当前会话的消息
                    state.list.items[chatIndex].messages = [];
                }
            }
            state.list.messageStatus = 'idle';
            state.list.messageError = null;
        },

        // 重置发送消息状态
        resetSendMessageStatus: (state) => {
            state.sendMessage.status = 'idle';
            state.sendMessage.error = null;
        },

        // 添加消息
        addMessage: (state, action) => {
            const conversationId = state.currentChat.conversationId;
            if (conversationId) {
                const chatIndex = state.list.items.findIndex((chat) => chat.conversation_id === conversationId);
                console.log(chatIndex, 'chatIndex');
                if (chatIndex !== -1) {
                    // 确保messages数组存在
                    if (!state.list.items[chatIndex].messages) {
                        state.list.items[chatIndex].messages = [];
                    }
                    // 添加消息到对应会话
                    state.list.items[chatIndex].messages.push(action.payload);

                    // 更新最后一条消息和消息数量
                    if (action.payload.role === 'user') {
                        state.list.items[chatIndex].last_message = action.payload.content;
                        if (state.list.items[chatIndex].message_count) {
                            state.list.items[chatIndex].message_count += 1;
                        } else {
                            state.list.items[chatIndex].message_count = 1;
                        }
                    }
                }
            }
        },

        // 更新消息
        updateMessage: (state, action) => {
            const { id, ...updates } = action.payload;
            const conversationId = state.currentChat.conversationId;

            if (conversationId) {
                const chatIndex = state.list.items.findIndex((chat) => chat.conversation_id === conversationId);

                if (chatIndex !== -1 && state.list.items[chatIndex].messages) {
                    const messageIndex = state.list.items[chatIndex].messages.findIndex((msg) => msg.id === id);

                    if (messageIndex !== -1) {
                        // 更新现有消息 - 确保完全替换内容以触发React更新
                        if (updates.content !== undefined) {
                            state.list.items[chatIndex].messages[messageIndex].content = updates.content;
                        }

                        // 更新其他字段
                        state.list.items[chatIndex].messages[messageIndex] = {
                            ...state.list.items[chatIndex].messages[messageIndex],
                            ...updates,
                        };

                        // 如果流式传输结束，更新发送消息状态
                        if (updates.is_streaming === false) {
                            state.sendMessage.status = 'succeeded';
                        }

                        // 如果是最后一条消息且是助手消息，更新会话的last_message
                        if (state.list.items[chatIndex].messages[messageIndex].role === 'assistant') {
                            state.list.items[chatIndex].last_message =
                                updates.content || state.list.items[chatIndex].messages[messageIndex].content;
                        }
                    } else {
                        // 如果找不到消息，尝试创建一个新消息
                        console.warn(`消息 ID ${id} 不存在，无法更新`);
                    }
                }
            }
        },

        // 添加资源
        setMessageResources: (state, action) => {
            const { messageId, resources } = action.payload;
            state.resources = {
                messageId,
                items: resources,
                status: 'succeeded',
                error: null,
            };
        },

        // 清除资源
        clearMessageResources: (state) => {
            state.resources = {
                messageId: null,
                items: [],
                status: 'idle',
                error: null,
            };
        },

        // 设置资源加载状态
        setResourcesLoading: (state, action) => {
            state.resources.status = 'loading';
            state.resources.messageId = action.payload;
            state.resources.error = null;
        },

        // 设置资源加载失败
        setResourcesError: (state, action) => {
            state.resources.status = 'failed';
            state.resources.error = action.payload;
        },

        // 更新特定会话的消息
        updateChatMessages: (state, action) => {
            const { conversationId, messages } = action.payload;
            const chatIndex = state.list.items.findIndex((chat) => chat.conversation_id === conversationId);

            if (chatIndex !== -1) {
                state.list.items[chatIndex].messages = messages;
                // 同时更新当前会话ID
                state.currentChat.conversationId = conversationId;
            }
        },
    },
    extraReducers: (builder) => {
        // 获取聊天列表
        builder
            .addCase(fetchChats.pending, (state) => {
                state.list.status = 'loading';
            })
            .addCase(fetchChats.fulfilled, (state, action) => {
                state.list.status = 'succeeded';
                console.log(action.payload, '当前list.items:', state.list.items);

                // 检查是否是追加模式
                if (action.payload.append) {
                    // 追加模式：将新结果添加到现有列表的前面
                    // 确保每个聊天项都有messages数组
                    const newResults = action.payload.results.map((chat) => ({
                        ...chat,
                        messages: chat.messages || [],
                    }));
                    state.list.items = [...newResults, ...state.list.items];
                } else {
                    // 替换模式：使用新结果替换整个列表
                    // 确保每个聊天项都有messages数组
                    state.list.items = (action.payload.results || []).map((chat) => ({
                        ...chat,
                        messages: chat.messages || [],
                    }));
                    state.list.total = action.payload.total;
                }

                state.list.error = null;
            })
            .addCase(fetchChats.rejected, (state, action) => {
                state.list.status = 'failed';
                state.list.error = action.payload || action.error.message;
            })

            // 创建聊天
            .addCase(createChat.pending, (state) => {
                state.operations.status = 'loading';
            })
            .addCase(createChat.fulfilled, (state, action) => {
                state.operations.status = 'succeeded';
                state.list.items.unshift(action.payload);
                state.list.total += 1;
                state.currentChat.data = action.payload;
                state.currentChat.status = 'succeeded';
            })
            .addCase(createChat.rejected, (state, action) => {
                state.operations.status = 'failed';
                state.operations.error = action.payload || action.error.message;
            })

            // 删除聊天
            .addCase(deleteChat.pending, (state) => {
                state.operations.status = 'loading';
            })
            .addCase(deleteChat.fulfilled, (state, action) => {
                state.operations.status = 'succeeded';
                // 更新聊天列表
                state.list.items = state.list.items.filter((chat) => chat.conversation_id !== action.payload);

                if (state.list.total > 0) {
                    state.list.total -= 1;
                }

                if (state.currentChat.data && state.currentChat.data.conversation_id === action.payload) {
                    state.currentChat.data = null;
                }
            })
            .addCase(deleteChat.rejected, (state, action) => {
                state.operations.status = 'failed';
                state.operations.error = action.payload || action.error.message;
            })

            // 更新聊天
            .addCase(updateChat.pending, (state) => {
                state.operations.status = 'loading';
            })
            .addCase(updateChat.fulfilled, (state, action) => {
                state.operations.status = 'succeeded';
                const index = state.list.items.findIndex((chat) => chat.id === action.payload.id);
                if (index !== -1) {
                    state.list.items[index] = action.payload;
                }
                if (state.currentChat.data && state.currentChat.data.id === action.payload.id) {
                    state.currentChat.data = action.payload;
                }
            })
            .addCase(updateChat.rejected, (state, action) => {
                state.operations.status = 'failed';
                state.operations.error = action.payload || action.error.message;
            })

            // 发送聊天消息
            .addCase(sendMessage.pending, (state) => {
                state.sendMessage.status = 'loading';
                state.sendMessage.error = null;
            })
            .addCase(sendMessage.fulfilled, (state, action) => {
                state.sendMessage.status = 'succeeded';
                // 更新消息列表
                const index = state.messages.items.findIndex(
                    (msg) => msg.content === action.payload.content && msg.sender === action.payload.sender
                );
                if (index === -1) {
                    state.messages.items.push(action.payload);
                }
            })
            .addCase(sendMessage.rejected, (state, action) => {
                state.sendMessage.status = 'failed';
                state.sendMessage.error = action.error.message;
            })

            // 处理创建聊天记录
            .addCase(createChatRecord.pending, (state) => {
                state.sendMessage.status = 'loading';
                state.sendMessage.error = null;
            })
            .addCase(createChatRecord.fulfilled, (state, action) => {
                // 更新状态以反映聊天已创建
                if (action.payload.conversation_id && !state.currentChat.data) {
                    // 设置当前聊天的会话ID
                    state.currentChat.data = {
                        conversation_id: action.payload.conversation_id,
                        // 其他信息将由流式更新填充
                    };
                }
                // 不再在这里添加消息，因为消息已经在thunk函数中添加
            })
            .addCase(createChatRecord.rejected, (state, action) => {
                state.sendMessage.status = 'failed';
                state.sendMessage.error = action.error.message;
            })

            // 处理创建会话
            .addCase(createConversation.pending, (state) => {
                state.createSession.status = 'loading';
                state.createSession.error = null;
            })
            .addCase(createConversation.fulfilled, (state, action) => {
                state.createSession.status = 'succeeded';
                state.createSession.sessionId = action.payload.conversation_id;

                // 当前聊天设置 - 使用与fetchConversationDetail相同的数据结构
                state.currentChat.data = {
                    conversation_id: action.payload.conversation_id,
                    datasets: action.payload.datasets || [],
                    // 添加其他必要的字段，确保与fetchConversationDetail返回的数据结构兼容
                    messages: [],
                    create_time: new Date().toISOString(),
                    update_time: new Date().toISOString(),
                };
                state.currentChat.status = 'succeeded';
                state.currentChat.error = null;
            })
            .addCase(createConversation.rejected, (state, action) => {
                state.createSession.status = 'failed';
                state.createSession.error = action.payload || action.error.message;
            })

            // 处理获取可用知识库
            .addCase(fetchAvailableDatasets.pending, (state) => {
                state.availableDatasets.status = 'loading';
                state.availableDatasets.error = null;
            })
            .addCase(fetchAvailableDatasets.fulfilled, (state, action) => {
                state.availableDatasets.status = 'succeeded';
                state.availableDatasets.items = action.payload || [];
                state.availableDatasets.error = null;
            })
            .addCase(fetchAvailableDatasets.rejected, (state, action) => {
                state.availableDatasets.status = 'failed';
                state.availableDatasets.error = action.payload || '获取可用知识库失败';
            })

            // 获取会话详情
            .addCase(fetchConversationDetail.pending, (state) => {
                state.currentChat.status = 'loading';
                state.currentChat.error = null;
            })
            .addCase(fetchConversationDetail.fulfilled, (state, action) => {
                if (action.payload) {
                    state.currentChat.status = 'succeeded';
                    state.currentChat.data = action.payload;
                } else {
                    state.currentChat.status = 'idle';
                    state.currentChat.data = null;
                }
            })
            .addCase(fetchConversationDetail.rejected, (state, action) => {
                state.currentChat.status = 'failed';
                state.currentChat.error = action.payload || action.error.message;
            });
    },
});

// 导出 actions
export const {
    resetOperationStatus,
    resetCurrentChat,
    setCurrentChat,
    resetMessages,
    resetSendMessageStatus,
    addMessage,
    updateMessage,
    setMessageResources,
    clearMessageResources,
    setResourcesLoading,
    setResourcesError,
    updateChatMessages,
} = chatSlice.actions;

// 导出 reducer
export default chatSlice.reducer;
