import { createSlice } from '@reduxjs/toolkit';
import { fetchNotifications, markNotificationRead, markAllNotificationsRead } from './notificationCenter.thunks';

const initialState = {
    notifications: [],
    unreadCount: 0,
    isConnected: false,
    loading: 'idle',
    error: null,
};

const notificationCenterSlice = createSlice({
    name: 'notificationCenter',
    initialState,
    reducers: {
        clearNotifications: (state) => {
            state.notifications = [];
            state.unreadCount = 0;
        },

        addNotification: (state, action) => {
            // 检查通知是否已存在
            const exists = state.notifications.some((n) => n.id === action.payload.id);
            if (!exists) {
                // 将新通知添加到列表的开头
                state.notifications.unshift(action.payload);
                // 如果通知未读，增加未读计数
                if (!action.payload.is_read) {
                    state.unreadCount += 1;
                }
            }
        },

        markNotificationAsRead: (state, action) => {
            const notification = state.notifications.find((n) => n.id === action.payload);
            if (notification && !notification.is_read) {
                notification.is_read = true;
                state.unreadCount = Math.max(0, state.unreadCount - 1);
            }
        },

        markAllNotificationsAsRead: (state) => {
            state.notifications.forEach((notification) => {
                notification.is_read = true;
            });
            state.unreadCount = 0;
        },

        setWebSocketConnected: (state, action) => {
            state.isConnected = action.payload;
        },

        removeNotification: (state, action) => {
            const notificationIndex = state.notifications.findIndex((n) => n.id === action.payload);
            if (notificationIndex !== -1) {
                const notification = state.notifications[notificationIndex];
                if (!notification.isRead) {
                    state.unreadCount = Math.max(0, state.unreadCount - 1);
                }
                state.notifications.splice(notificationIndex, 1);
            }
        },
    },
    extraReducers: (builder) => {
        builder
            .addCase(fetchNotifications.pending, (state) => {
                state.loading = 'loading';
            })
            .addCase(fetchNotifications.fulfilled, (state, action) => {
                state.loading = 'succeeded';
                state.notifications = action.payload;
                // 计算未读通知数量
                state.unreadCount = action.payload.filter((notification) => !notification.is_read).length;
            })
            .addCase(fetchNotifications.rejected, (state, action) => {
                state.loading = 'failed';
                state.error = action.payload?.message || 'Failed to fetch notifications';
            })
            .addCase(markNotificationRead.fulfilled, (state, action) => {
                const notification = state.notifications.find((n) => n.id === action.payload.id);
                if (notification && !notification.is_read) {
                    notification.is_read = true;
                    state.unreadCount = Math.max(0, state.unreadCount - 1);
                }
            })
            .addCase(markAllNotificationsRead.fulfilled, (state) => {
                state.notifications.forEach((notification) => {
                    notification.is_read = true;
                });
                state.unreadCount = 0;
            });
    },
});

export const {
    clearNotifications,
    addNotification,
    markNotificationAsRead,
    markAllNotificationsAsRead,
    setWebSocketConnected,
    removeNotification,
} = notificationCenterSlice.actions;

export default notificationCenterSlice.reducer;
